package de.intevation.flys.exports;

import org.apache.log4j.Logger;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.title.TextTitle;
import org.jfree.data.xy.XYSeries;

import org.w3c.dom.Document;

import de.intevation.artifacts.Artifact;

import de.intevation.artifactdatabase.state.Facet;

import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.WKms;

import de.intevation.flys.utils.DataUtil;


/**
 * An OutGenerator that generates w differences curves.
 */
public class WDifferencesCurveGenerator
extends      LongitudinalSectionGenerator
implements   FacetTypes
{
    /** The logger that is used in this generator. */
    private static Logger logger =
        Logger.getLogger(WDifferencesCurveGenerator.class);

    public static final String I18N_CHART_TITLE =
        "chart.w_differences.title";

    public static final String I18N_CHART_SUBTITLE =
        "chart.w_differences.subtitle";

    public static final String I18N_2YAXIS_LABEL =
        "chart.w_differences.yaxis.second.label";

    // TODO proper i18n.
    public static final String I18N_CHART_TITLE_DEFAULT  = "W-Differenzen";
    public static final String I18N_XAXIS_LABEL_DEFAULT  = "km";
    public static final String I18N_YAXIS_LABEL_DEFAULT  = "W [NN + m]";


    /**
     * Add a subtitle to Chart.
     * @param chart Chart to add subtitle to.
     */
    @Override
    protected void addSubtitles(JFreeChart chart) {
        String subtitle = msg(I18N_CHART_SUBTITLE, "");
        chart.addSubtitle(new TextTitle(subtitle));
    }


    /**
     * Add (themed) data for chart generation.
     */
    public void doOut(Artifact artifact, Facet facet, Document attr) {
        String name = facet.getName();

        logger.debug("WDifferencesCurveGenerator.doOut: " + name);

        if (name == null) {
            logger.error("No facet name for doOut(). No output generated!");
            return;
        }

        FLYSArtifact flys = (FLYSArtifact) artifact;
        Facet        f    = flys.getNativeFacet(facet);

        if (f == null) {
            return;
        }

        if (name.equals(W_DIFFERENCES)) {
            doWDifferencesOut((WKms) f.getData(artifact, context), f.getDescription(), attr);
        }
        else {
            logger.warn("Unknown facet name: " + name);
            return;
        }
    }

    // TODO reenable inversion.
    /**
     * Add items to dataseries which describes the differences.
     */
    protected void doWDifferencesOut(WKms wkms, String seriesName, Document theme) {
        logger.debug("WDifferencesCurveGenerator.doWDifferencesOut");
        if (wkms == null) {
            logger.warn("No data to add to WDifferencesChart.");
            return;
         }

        int size = wkms.size();
        XYSeries series = new StyledXYSeries(seriesName, theme);

        if (logger.isDebugEnabled()) {
            if (wkms.size() > 0) {
                logger.debug("Generate series: " + series.getKey());
                logger.debug("Start km: " + wkms.getKm(0));
                logger.debug("End   km: " + wkms.getKm(size-1));
                logger.debug("Values  : " + size);
            }
        }

        for (int i = 0; i < size; i++) {
            series.add(wkms.getKm(i), wkms.getW(i));
        }

        addFirstAxisSeries(series);
        if (DataUtil.guessWaterIncreasing(wkms)) {
            setInverted(true);
        }
    }


    /**
     * Get name of series (displayed in legend).
     * @return name of the series.
     */
    protected String getSeriesName(WKms wqkms, String mode) {
        String name   = wqkms.getName();
        String prefix = (name != null && name.indexOf(mode) >= 0)
                      ? null
                      : mode;

        return (prefix != null && prefix.length() > 0)
                ? prefix + "(" + name +")"
                : name;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
